@@ -1,3 +1,4 @@
+#!/usr/bin/python
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -24,15 +25,13 @@ import argparse
 from trigger import utils
 from trigger import config
 from trigger import extension
-from trigger.driver import LockDriverError
-from trigger.driver import SyncDriverError
-from trigger.driver import ServiceDriverError
+from driver import LockDriverError
+from driver import SyncDriverError
+from driver import ServiceDriverError
+from trigger.config import ConfigurationError
 from datetime import datetime
 from git import GitCommandError
-from git.repo import Repo
 
-CONF = config.CONF
-CONF.register_drivers()
 LOG = config.LOG
 
 
@@ -48,14 +47,24 @@ class TriggerError(Exception):
 
 class Trigger(object):
 
-    def __init__(self):
-        self._lock_driver = CONF.drivers['lock-driver']
-        self._sync_driver = CONF.drivers['sync-driver']
-        self._service_driver = CONF.drivers['service-driver']
+    def __init__(self, conf):
+        self.conf = conf
+        self._lock_driver = self.conf.drivers['lock-driver']
+        self._sync_driver = self.conf.drivers['sync-driver']
+        self._service_driver = self.conf.drivers['service-driver']
 
     def do_start(self, args):
-        if self._lock_driver.check_lock(args):
-            message = 'A deployment has already been started for this repo.'
+        """
+        Start a deployment for this repository and hold the deployment lock.
+        """
+        lock_info = self._lock_driver.check_lock(args)
+        if lock_info:
+            if lock_info['user']:
+                message = ('A deployment has already been started for this'
+                           ' repo by {0}.').format(lock_info['user'])
+            else:
+                message = ('A deployment has already been started for this'
+                           'repo.')
             raise TriggerError(message, 100)
         try:
             self._lock_driver.add_lock(args)
@@ -80,16 +89,32 @@ class Trigger(object):
                action='store_true',
                default=False,
                help='Do not reset the working tree to the start tag.')
+    @utils.arg('--force',
+               dest='force',
+               action='store_true',
+               default=False,
+               help='Abort even if another user started this deployment.')
     def do_abort(self, args):
-        if not self._lock_driver.check_lock(args):
+        """
+        Abort this deployment, resetting the local repository back to the
+        start tag.
+        """
+        lock_info = self._lock_driver.check_lock(args)
+        if not lock_info:
             message = 'There is no deployment to abort.'
             raise TriggerError(message, 130)
+        if lock_info['user'] != self.conf.config['user.name']:
+            if not args.force:
+                message = ('{0} started this deployment, use --force to'
+                           ' abort that deployment.').format(lock_info['user'])
+                raise TriggerError(message, 132)
         if not args.noreset:
             try:
                 start_tag = self._get_latest_tag('start')
                 if start_tag:
-                    CONF.repo.head.reset(commit=start_tag.commit, index=True,
-                                          working_tree=True)
+                    self.conf.repo.head.reset(commit=start_tag.commit,
+                                              index=True,
+                                              working_tree=True)
                 else:
                     LOG.warning('Could not find a start tag to reset to.')
             except GitCommandError:
@@ -101,11 +126,20 @@ class Trigger(object):
             raise TriggerError(e.message, 131)
         LOG.info('Deployment aborted.')
 
+    @utils.arg('--force',
+               dest='force',
+               action='store_true',
+               default=False,
+               help='Force a sync even if nothing changed locally.')
     def do_sync(self, args):
+        """
+        Synchronize the current state of the local repository to all
+        deployment targets.
+        """
         if not self._lock_driver.check_lock(args):
             message = 'A deployment has not been started.'
             raise TriggerError(message, 160)
-        if CONF.repo.is_dirty():
+        if self.conf.repo.is_dirty():
             message = ('The repository is dirty. Please commit or revert any'
                        ' uncommitted changes.')
             raise TriggerError(message, 161)
@@ -115,22 +149,43 @@ class Trigger(object):
             self._sync_driver.sync(tag, args)
         except SyncDriverError as e:
             raise TriggerError(e.message, 163)
+        try:
+            self._lock_driver.remove_lock(args)
+        except LockDriverError as e:
+            raise TriggerError(e.message, 131)
+        # TODO (ryan-lane): display amount of time of deployment
+        LOG.info('Deployment finished.')
+
+    def do_finish(self, args):
+        """
+        Finish the deployment and release the deloyment lock. This is called
+        automatically when sync successfully exits.
+        """
+        if not self._lock_driver.check_lock(args):
+            message = 'A deployment has not been started.'
+            raise TriggerError(message, 160)
+        try:
+            self._lock_driver.remove_lock(args)
+        except LockDriverError as e:
+            raise TriggerError(e.message, 131)
+        # TODO (ryan-lane): display amount of time of deployment
+        LOG.info('Deployment finished.')
 
     def _write_tag(self, tag_type):
         #TODO: use a tag driver
         timestamp = datetime.now().strftime('%Y%m%d-%H%M%S')
         try:
-            tag_format = '{0}-{1}-{2}'.format(CONF.config['repo-name'],
+            tag_format = '{0}-{1}-{2}'.format(self.conf.config['repo-name'],
                                               tag_type,
                                               timestamp)
-            return CONF.repo.create_tag(tag_format)
+            return self.conf.repo.create_tag(tag_format)
         except GitCommandError:
             message = 'Failed to write the {0} tag.'.format(tag_type)
             raise TriggerError(message, 190)
 
     def _get_latest_tag(self, tag_type):
         tag_filter = '-{0}-'.format(tag_type)
-        tags = filter(lambda k: tag_filter in k.name, CONF.repo.tags)
+        tags = filter(lambda k: tag_filter in k.name, self.conf.repo.tags)
         if len(tags) > 0:
             return tags[-1]
         else:
@@ -141,18 +196,23 @@ class Trigger(object):
                help='Service action to take: stop|start|restart|reload')
     @utils.arg('--batch',
                dest='batch',
+               default='10%',
                help='Number or percentage of targets to target for'
                     ' batch processing.')
     def do_service(self, args):
         """
         Manage the service associated with this repository.
         """
+        # TODO (ryan-lane): Make this more extendable and have the help
+        #                   report implemented functions.
         try:
-            getattr(self.service_driver, args.action)(args)
-        except AttributeError, NotImplementedError:
+            getattr(self._service_driver, args.action)(args)
+        except (AttributeError, NotImplementedError):
             msg = '{0} is not an action implemented by this service driver.'
             msg = msg.format(args.action)
             raise TriggerError(msg, 200)
+        except ServiceDriverError as e:
+            raise TriggerError(e.message, 201)
 
     @utils.arg('command', metavar='<subcommand>', nargs='?',
                help='Display help for <subcommand>.')
@@ -164,8 +224,8 @@ class Trigger(object):
             if args.command in self.subcommands:
                 self.subcommands[args.command].print_help()
             else:
-                raise exc.CommandError("'%s' is not a valid subcommand" %
-                                       args.command)
+                msg = "'{0}' is not a valid subcommand".format(args.command)
+                raise TriggerError(msg, 1)
         else:
             self.parser.print_help()
 
@@ -192,11 +252,13 @@ class Trigger(object):
             yield name, module
 
     def _get_base_parser(self):
+        epilog = ('See "{0} help COMMAND" '
+                  'for help on a specific command.')
+        epilog = epilog.format(self.command_name)
         parser = argparse.ArgumentParser(
-            prog="trigger",
+            prog=self.command_name,
             description=__doc__.strip(),
-            epilog='See "trigger help COMMAND" '
-                   'for help on a specific command.',
+            epilog=epilog,
             add_help=False
         )
 
@@ -225,30 +287,29 @@ class Trigger(object):
             arguments = getattr(callback, 'arguments', [])
 
             subparser = subparsers.add_parser(command,
-                help=action_help,
-                description=desc,
-                add_help=False
-            )
+                                              help=action_help,
+                                              description=desc,
+                                              add_help=False)
             subparser.add_argument('-h', '--help',
-                action='help',
-                help=argparse.SUPPRESS,
-            )
+                                   action='help',
+                                   help=argparse.SUPPRESS)
             self.subcommands[command] = subparser
             for (args, kwargs) in arguments:
                 subparser.add_argument(*args, **kwargs)
             subparser.set_defaults(func=callback)
 
-    def main(self, argv):
+    def main(self, name, argv):
         # TODO (ryan-lane): Add novaclient's model for hooks
+        name = os.path.basename(name)
+        self.command_name = name[4:]
         self.extensions = self._discover_extensions()
         self.parser = self._get_subcommand_parser()
         args = self.parser.parse_args(argv)
 
-        if args.func == self.do_help:
-            self.do_help(args)
-            return 0
-
         try:
+            if args.func == self.do_help:
+                self.do_help(args)
+                return 0
             args.func(args)
         except TriggerError as e:
             LOG.error(e.message)
@@ -256,8 +317,13 @@ class Trigger(object):
 
 
 def main():
-    trigger = Trigger()
-    trigger.main(sys.argv[1:])
+    try:
+        conf = config.Configuration()
+    except ConfigurationError as e:
+        LOG.error(e.message)
+        raise SystemExit(e.errorno)
+    trigger = Trigger(conf)
+    trigger.main(sys.argv[0], sys.argv[1:])
 
 if __name__ == "__main__":
     main()
